////////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2003-2006 Adobe Macromedia Software LLC and its licensors.
//  All Rights Reserved. The following is Source Code and is subject to all
//  restrictions on such code as contained in the End User License Agreement
//  accompanying this product.
//
////////////////////////////////////////////////////////////////////////////////

package src.main
{
import mx.automation.IAutomationEnvironment;
import mx.automation.IAutomationClass;
import mx.automation.IAutomationObject;
import mx.automation.IAutomationMethodDescriptor;
import mx.automation.IAutomationEventDescriptor;
import mx.automation.IAutomationPropertyDescriptor;
import mx.utils.StringUtil;
import src.main.utilities.XMLParsing;

/**
 * Custom Environment class (Used with Adobe's approval).
 * @author Adobe
 */
public class CustomEnvironment implements IAutomationEnvironment
{
	//--------------------------------------------------------------------------
	//
	//  Constructor
	//
	//--------------------------------------------------------------------------

    /**
     *  @private
	 */
	public function CustomEnvironment(source:XML)
    {
		super();
        
        // Parse the XML, and populate the objects
        var parser:XMLParsing = new XMLParsing();
		var outputXML:XML = parser.parse(source);
        
        fillObjects(outputXML);
    }

	//--------------------------------------------------------------------------
	//
	//  Variables
	//
	//--------------------------------------------------------------------------
	
    /**
	 *  @private
	 */
    private var className2automationClass:Object = {};
    
    /**
	 *  @private
	 */
	private var automationClassName2automationClass:Object = {};
	
	//--------------------------------------------------------------------------
	//
	//  Methods
	//
	//--------------------------------------------------------------------------
	
    /**
     *  @private
	 */
    private function fillObjects(source:XML):void
    {
    	for (var i:Object in source.ClassInfo)
        {
            // populate the class/automationClass map
            var classInfoXML:XML = source.ClassInfo[i];
            var automationClassName:String = classInfoXML.@Name.toString();
            var superClassName:String = 
                classInfoXML.@Extends && classInfoXML.@Extends.length != 0 ?
				classInfoXML.@Extends.toString() :
				null;
            var automationClass:CustomAutomationClass =
				new CustomAutomationClass(automationClassName, superClassName);
            automationClassName2automationClass[automationClassName] =
				automationClass;
            
            for (var j:Object in classInfoXML.Implementation)
            {
                var implementationXML:XML = classInfoXML.Implementation[j];
                var realClassName:String =
					implementationXML.@Class.toString().replace("::", ".");
                className2automationClass[realClassName] = automationClass;
                automationClass.addImplementationClassName(realClassName);
            }

            // for each Method
            for (var k:Object in classInfoXML.Methods.Method)
            {
                var methodXML:XML = classInfoXML.Methods.Method[k];
			
                var automationMethodName:String = methodXML.@Name.toString();
                
                var methodType:String =
					methodXML.Implementation.@Type.toString();
				
                if (methodType)
                {
                    var parameters:Array = [];
                    for (var m:Object in methodXML.Parameter)
                    {
                        var parameterXML:XML = methodXML.Parameter[m];
                        var parameterName:String = parameterXML.@Name.toString();
                        var parameterType:String =
							parameterXML.PropertyType.@Type.toString().toLowerCase();
                        var pd:CustomAutomationPropertyDescriptor = 
                            new CustomAutomationPropertyDescriptor(
								parameterName, true, true, parameterType);
                        parameters.push(pd);
                    }
                    
                    var returnType:String =
						methodXML.ReturnValueType.PropertyType.@Type.toString();

					var md:IAutomationMethodDescriptor = new CustomAutomationMethodDescriptor(
							automationMethodName, methodType, returnType,parameters);
	                automationClass.addMethod(md);

                }
            }
            
            // for each Event
            for (var q:Object in classInfoXML.Events.Event)
            {
                var eventXML:XML = classInfoXML.Events.Event[q];
			
                var automationEventName:String = eventXML.@Name.toString();
                var eventClassName:String =
					eventXML.Implementation.@Class.toString();
                eventClassName = eventClassName.replace("::", ".");
                var eventType:String =
					eventXML.Implementation.@Type.toString();

                if (eventType)
                {
	                var events:Array = [];
	                for (var n:Object in eventXML.Property)
	                {
	                    var propertyEventXML:XML = eventXML.Property[n];
	                    var propertyEventName:String = propertyEventXML.@Name.toString();
	                    var propertyEventType:String =
							propertyEventXML.PropertyType.@Type.toString().toLowerCase();
	                    var defaultValue:String = 
	                        propertyEventXML.@DefaultValue.length() > 0 ?
							propertyEventXML.@DefaultValue.toString() :
							null;
	                    var pd1:CustomAutomationPropertyDescriptor = 
	                        new CustomAutomationPropertyDescriptor(
								propertyEventName, true, true, eventType, defaultValue);
	                    events.push(pd1);
	                }
                    
					var ed:IAutomationEventDescriptor = new CustomAutomationEventDescriptor(
							automationEventName, eventClassName,
                            eventType, events);
                    automationClass.addEvent(ed);
                }
            }
            
            
            // for each Property
            for (var p:Object in classInfoXML.Properties.Property)
            {
                var propertyXML:XML = classInfoXML.Properties.Property[p];
				var propName:String = propertyXML.@Name.toString();
				//trace("------ PROP-NAME-------| " + propName);
				var propType:String =
					propertyXML.PropertyType.@Type.toString().toLowerCase();
                var pd2:CustomAutomationPropertyDescriptor = 
					new CustomAutomationPropertyDescriptor(
						propName,
                        propertyXML.@ForDescription.toString() == "true",
                        propertyXML.@ForVerification.toString() == "true",
                        propType);
                automationClass.addPropertyDescriptor(pd2);
            }
        }
    }
    
    /**
     *  @private
	 */
    public function getAutomationClassByInstance(
						obj:IAutomationObject):IAutomationClass
    {
        var result:IAutomationClass = findClosestAncestor(obj);
        if (! result)
		{
             return null;
		}
        return result;
    }

    /**
     *  @private
	 */
    public function getAutomationClassByName(
						automationClass:String):IAutomationClass
    {
        return CustomAutomationClass(
			automationClassName2automationClass[automationClass]);
    }
    
    /**
     *  Finds the closest ancestor to this object about which information was 
     *  passed in the environment XML.
     *  @private
     */
    private function findClosestAncestor(obj:Object):IAutomationClass
    {   	
        var className:String = CustomAutomationClass.getClassName(obj);
        if (className in className2automationClass)
            return className2automationClass[className];
		
        var ancestors:Array = findAllAncestors(obj);
        if (ancestors.length != 0)
        {
            className2automationClass[className] = ancestors[0];
            return className2automationClass[className];
        }
        else
		{
            return null;
		}
    }

    /**
	 *  @private
	 */
    private function findAllAncestors(obj:Object):Array
    {
        var result:Array = [];
        
		for (var i:String in className2automationClass)
        {
		    var realClass:Class = 
                          CustomAutomationClass.getDefinitionFromObjectDomain(obj,i);
            if (realClass && obj is realClass)
                result.push(className2automationClass[i]);
        }
        
		result.sort(sortAncestors);
        
		return result;
    }
    
    /**
	 *  @private
	 */
    private function sortAncestors(a:IAutomationClass, b:IAutomationClass):int
    {
        var superClass:IAutomationClass;
        var x:String = a.superClassName;
        while (x)
        {
            if (x == b.name)
                return -1;
            superClass = getAutomationClassByName(x);
            x = superClass.superClassName;
        }
        
		x = b.superClassName;
        while (x)
        {
            if (x == a.name)
                return 1;
            superClass = getAutomationClassByName(x);
            x = superClass.superClassName;
        }
        
		// Bad things will happen if we return 0... maybe throw an error?
        return 0;
    }
}

}
