/*
 * FunFX is an adapter used to enable the Funfx framework written in Ruby to
 * functionally test Flex applications. It uses ExternalInterface to make some
 * methods externally available.
 *
 * Copyright (c) 2007 Peter Nicolai Motzfeldt.
 * Licensed under open source. No warranty is provided
 */
package src.main
{
	import flash.display.Sprite;
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.InteractiveObject;
	
	import flash.events.Event;
	import flash.external.ExternalInterface;
	import flash.utils.getDefinitionByName;
	
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	
	import mx.automation.events.AutomationRecordEvent;
	import mx.automation.events.AutomationReplayEvent;
	import mx.automation.IAutomationManager;
	import mx.automation.IAutomationObject;
	import mx.automation.AutomationID;
	import mx.automation.Automation;
	
	import mx.core.Application;
	import mx.core.UIComponent;
	
	import mx.controls.listClasses.IListItemRenderer;
	
	import mx.events.FlexEvent;
	
	import mx.rpc.http.HTTPService;
	import mx.containers.TabNavigator;
	import mx.core.UITextField;
	import mx.managers.SystemManager;
	import mx.controls.treeClasses.TreeItemRenderer;
	
	[Mixin]
	/**
	 * Adapter that enables the Ruby framework Funfx to functionally test flex applications.
	 * 
	 * @author Peter Nicolai Motzfeldt
	 * 
	 */
	public class FunFX extends Sprite
	{
		private static var funFX:FunFX;
		private var myLoader:URLLoader;
		private static var _root:DisplayObject;
		
		public function FunFX(root:DisplayObject)
	    {
	       	super();
			root.addEventListener(FlexEvent.APPLICATION_COMPLETE, applicationCompleteHandler);
			_root = root;
			
			if(ExternalInterface.available)
			{
				ExternalInterface.addCallback("addevent", catchEvent);
				ExternalInterface.addCallback("checkProperty", catchProperty);
				ExternalInterface.addCallback("checkComponent", checkComponent);
				ExternalInterface.addCallback("beginRecording", beginRecording);
				ExternalInterface.addCallback("endRecording", endRecording);
				ExternalInterface.addCallback("getTabularData", getTabularDataValues);
				ExternalInterface.addCallback("getTabularDataValue", getTabularDataValue);
				ExternalInterface.addCallback("firstVisibleRow", getFirstVisibleRow);
				ExternalInterface.addCallback("lastVisibleRow", getLastVisibleRow);
				ExternalInterface.addCallback("numRows", getNumRows);
				ExternalInterface.addCallback("numColumns", getNumColumns);
				ExternalInterface.addCallback("columnNames", getColumnNames);
				ExternalInterface.addCallback("ready", applicationReady);
				
				ExternalInterface.addCallback("getChildren", getChildren);
				ExternalInterface.addCallback("isSynchronized", isSynchronized);
				
			}
			else
				trace("Error: Dette gikk ikke");

	    }
	    
	    /**
	     * Init method that initializes the adapter class when added as a library to 
	     * a Flex application.
	     * 
	     * @param root The Flex application
	     * 
	     */
	    public static function init(root:DisplayObject):void
	    {
	    	if(!funFX)
		    {
		    	_root = root;
		    	funFX = new FunFX(root);
	    	}
		}
		
		/**
		 * Fires when the Flex application is loaded complete
		 * Then it uses the method @see setTestingEnvironment to set the testing environment.
		 * 
		 * @param event The Flex event, saying the Flex application is complete
		 * 
		 */
		private function applicationCompleteHandler(event:FlexEvent):void
		{
			_root.removeEventListener(FlexEvent.APPLICATION_COMPLETE, applicationCompleteHandler);

			funFX = this;
			
			// Load environment XML
	       	var te:String = "c:\Adobe\libs\AutomationGenericEnv.xml";
	       	setTestingEnvironment(te);
		}
		
		/**
		 * Sets the testing environment, with a XML file that describes all the
		 * Flex components. It uses the private method xmlLoaded and the classes created by Adobe.
		 * 
		 * @param te Testing environment filename
		 * 
		 */
		public function setTestingEnvironment(te:String):void
	    {
	    	// Loading the XML file
	    	var myXMLURL:URLRequest = new URLRequest(te);
			myLoader = new URLLoader(myXMLURL);
			myLoader.addEventListener("complete", xmlLoaded);
	    }
	    
	    private function xmlLoaded(evtObj:Event):void
		{ 
	        var source:XML = XML(myLoader.data);
	        
	        // CustomEnvironment class implements IAutomationEnvironment interface
	        automationManager.automationEnvironment = new CustomEnvironment(new XML(source));
		}
		
		/**
		 * Checks wether the application root is ready/initialized or not
		 * 
		 * @param str With ExternalInterface the method must provide some arguement
		 * @return A Boolean value indicating if the application is ready or not
		 * 
		 */
		public static function applicationReady(str:String):Boolean
		{
			return (Application.application.root != null);
		}
		
		
		/**
		 * Helps the Funfx framework to figure out if the
		 * object exist in the application. Uses the methods findObject and 
		 * createID.
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @return A boolean value indicating if the object exists or not
		 * @see findObject
		 * @see createID
		 * @see mx.automation.AutomationID
		 */
		public function checkComponent(parentID:String, name:String):*
		{
			var obj:IAutomationObject = findObject(parentID,name);
			if(obj == null) return false;
			else return createID(obj);
		}
		
		/**
		 * A help method to create the ID for the object. It is created to ease testing.
		 * @param obj The display object
		 * @return The id created with the help from AutomationID
		 * 
		 * @see mx.automation.AutomationID
		 */
		protected function createID(obj:IAutomationObject):String
		{
			return automationManager.createID(obj).toString();
		}
		
		/**
		 * Checks the value of an objects property.
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @param property the property of an object the framework wants to check its value
		 * @return  the value of the property
		 * 
		 * @see mx.automation.AutomationID
		 */
		public function catchProperty(objectID:String, property:String):*
		{	
			var object:IAutomationObject = IAutomationObject(resolveObjFromID(objectID));
			
			if(object == null) return null;
			
			var propertyResult:Array = automationManager.getProperties(object, property.split(","));

			return propertyResult[0];
		}
		
		
		/**
		 * Method that resolves the object from the cerated ID
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @return The object resolved from the ID.
		 * 
		 * @see mx.automation.AutomationID
		 */
		public function resolveObjFromID(objectID:String):UIComponent
		{
			var objArray:Array = resolveID(objectID);
			return UIComponent(objArray[0]);
		}
		
		/**
		 * Helper method for resolving the object from the ID. Made to ease testing.
		 * 
		 * @param objectID The id of a object created with mx.automation.AutomationID
		 * @return Array
		 * 
		 * @see mx.automation.AutomationID
		 */
		protected function resolveID(objectID:String):Array
		{
			return automationManager.resolveID(AutomationID.parse(objectID));
		}
		
		
		/**
		 * Uses the objectTree string, which is a comma separated
		 * string, to go through the display object hierarchy to find the wanted object.
		 * It uses the methods startSearch and performSearch to achieve this goal.
		 * 
		 * @param parentID The id of the parent object, String/AutomationID
		 * @param name The name of the object
		 * @return The object found, if noe object found it returns null
		 * 
		 * @see startSearch
		 * @see performSearch
		 * @see mx.automation.AutomationID
		 */
		 
		public function findObject(parentID:String, name:String):IAutomationObject
		{
			if(name == null) return null;
			if(parentID == null)
				return startSearch(name);
			else
				return performSearch(new Array(resolveObjFromID(parentID)), name);
		}
		
		
		/**
		 * A helper method that is used to find the first display object from
		 * the System manager. After the first object is discovered, it uses the performSearch method
		 * to locate the next objects
		 * 
		 * @param name The name of the object
		 * @return The object found or null
		 * 
		 * @see performSearch
		 */
		protected function startSearch(name:String):UIComponent
		{
			var parent:DisplayObjectContainer = DisplayObjectContainer(applicationRoot);
			var openList:Array = new Array();
			for(var i:int=0; i<parent.numChildren; i++)
			{
				try
				{
					openList.push(UIComponent(parent.getChildAt(i)));
				}
				catch(e:Error)
				{}
			}
			return performSearch(openList, name);
		}
		
		/**
		 * Uses a breadth first algorithm to find the object. The closest
		 * objects are put in an openlist and the investigated items that are not the one
		 * searched for are put in a closed list. It moves down in the hierarchy until the object
		 * is found. The method checkName is used to see if the object is correct or not.
		 * 
		 * @param openList The list of objects not investigated
		 * @param name The name of the object
		 * @return The object found, or null if not found.
		 * 
		 * @see checkName
		 */
		protected function performSearch(openList:Array, name:String):UIComponent
		{
			var closedList:Array = new Array();
			while(openList.length != 0)
			{
				var child:UIComponent  = UIComponent(openList.shift());
 				if(checkName(child, name)) return child;
 				
 				for(var i:int=0; i<child.numAutomationChildren; i++)
 				{
 					try
					{
 						openList.push(UIComponent(child.getAutomationChildAt(i)));
 					}
	 				catch(e:Error)
	 				{}
 				}
 				
 				closedList.push(child);
 			}
 			return null;
 		}
		
		/**
		 * Takes a display object and a name, and checks different values
		 * on the object to see if it matches the name.
		 * 
		 * @param obj The display object
		 * @param name The name of the desired display object
		 * @return Indicating that the display object is correct or not
		 * 
		 */
		public function checkName(obj:UIComponent, name:String):Boolean
		{
			
			if(obj.id == name) return true;
			if(obj.name == name) return true;
			if(obj.automationName == name) return true;
			return false;
		}
		
		/**
		 * The method the testing framework (Ruby implemented Funfx) access to perform events.
		 * It receives the object and the event that will be performed on the object. It creates the event and if the
		 * properties argument is set, it uses the method setProperties to merge the properties of the event.
		 * The event and object is sent to the method startPlayback.
		 * 
		 * @param event The event eg. flash.events::MouseEvent
		 * @param eventtype The type of event eg. Click
		 * @param objectID The id of the object, String/AutomationID
		 * @param properties The properties of the event, an XML object
		 * @return The error message if one is created, or null if the replay was successfull.
		 * 
		 * @see setProperties
		 * @see startPlayback
		 * @see mx.automation.AutomationID
		 */
		public function catchEvent(eventType:String, eventClass:String, objectID:String, properties:Object=null):String
		{
			var object:UIComponent = resolveObjFromID(objectID);

			if(object == null) return "No such object";
			else if(!Automation.automationManager.isSynchronized(object)) return "Not synchronized";
			
			try
			{
				var tempEvent:Class = getDefinitionByName(eventClass) as Class;
				var replayEvent:* = new tempEvent(eventType);
				
				if(properties != null)
				{
					var error:String = setProperties(replayEvent, object, properties);
					if(error == null)
						startPlayback(object, replayEvent);
					else
						return error;
				}
				else
					startPlayback(object, replayEvent);
					
			}
			catch(eventError:Error)
			{	
				return eventError.message;	
			}
			
			return null;
		}
		
		/**
		 * Merges the properties added by the framework to the actual event
		 * 
		 * @param replayEvent The event that will be replayed
		 * @param object The object the event will be performed on
		 * @param properties The event properties, an XML object
		 * @return The error message, if one is created, or null if it was successful.
		 * 
		 */
		public function setProperties(replayEvent:*, object:UIComponent, properties:Object):String
		{
			var xml:XML = new XML(properties);
			for (var i:Object in xml.Property)
    		{
    			var propertyXML:XML = xml.Property[i];
							
    			/**
    			* Checks wheter the properties default value is not false
    			*/
    			if(propertyXML.@DefaultValue != "false")
    			{
    				var type:String = propertyXML.PropertyType[0].@Type;
    				var tempClass:Class;
    				
    				/**
    				* Checks if the property type is Event, if so creates an instance of the
    				* type of Event stated in the default value 
    				*/
    				if(type == "Event")
    				{
    					if(propertyXML.@DefaultValue != "null")
    					{
    						tempClass = getDefinitionByName(propertyXML.@DefaultValue) as Class;
    						replayEvent[propertyXML.@Name.toString()] = new tempClass(""); //Må gjøres noe med, er en mouse or keyboard event uten noen type!!!
    					}
    					else
    						return "Cannot create an event of the value <" + propertyXML.@DefaultValue + "> for the property <" + propertyXML.@Name.toString() + ">";
    				}
    				else
    				{
    					/**
    					* If the type is not Event, it does some special if the name is:
    					* - itemRenderer
    					* - draggedItem
    					* - relatedObject
    					*
    					* With these properties, the adapter needs to iterate over the 
    					* children to the object and see if the automationValue is equal
    					* the default value.
    					*/
    					var check:Boolean = false;
    					var count:Number;
    					tempClass = getDefinitionByName(type) as Class;
    					if(propertyXML.@Name == "itemRenderer" || propertyXML.@Name == "dropParent")
    					{
    						for (count = 0; count < object.numAutomationChildren; count++)
    						{
    							if(object.getAutomationChildAt(count).automationValue == propertyXML.@DefaultValue.toString())
    							{
    								replayEvent[propertyXML.@Name.toString()] = IListItemRenderer(object.getAutomationChildAt(count));
    								check = true;
    								break;
    							}
    						}
    						if(!check) return "No such itemRenderer <" + propertyXML.@DefaultValue.toString() + "> on the object <" + object.automationName + ">";
    					}
    					else if(propertyXML.@Name == "draggedItem")
    					{
    						if(propertyXML.@DefaultValue != "")
    						{
        						for (count = 0; count < object.numAutomationChildren; count++)
        						{
        							if(object.getAutomationChildAt(count).automationValue == propertyXML.@DefaultValue.toString())
        							{
        								replayEvent.draggedItem = object.getAutomationChildAt(count);
        								check = true;
        								break;
        							}	
        						}
        						if(!check) return "No draggable item contains the value <" + propertyXML.@DefaultValue.toString() + "> on the object with the name <" + object.automationName + ">";
    						}
    					}
    					else if(propertyXML.@Name == "relatedObject")
    					{
    						if(propertyXML.@DefaultValue != "")
    						{
        						for (count = 0; count < object.numAutomationChildren; count++)
        						{
        							if(object.getAutomationChildAt(count).automationValue == propertyXML.@DefaultValue.toString())
        							{
										if(object is TabNavigator)
											replayEvent[propertyXML.@Name.toString()] = TabNavigator(object).getTabAt(count);
										else
											replayEvent[propertyXML.@Name.toString()] = InteractiveObject(object.getAutomationChildAt(count));
        								check = true;
        								break;
        							}
        						}
        						if(!check) return "No such relatedObject <" + propertyXML.@DefaultValue.toString() + "> on the object <" + object.automationName + ">";
    						}
    						else
    						{
    							return "No such relatedObject <" + propertyXML.@DefaultValue.toString() + "> on the object <" + object.automationName + ">";
    						}
    					}
    					else if(type != "Date")
    						replayEvent[propertyXML.@Name.toString()] = new tempClass(propertyXML.@DefaultValue);
    					else
    					{
    						var tempDate:Array = String(propertyXML.@DefaultValue).split(".");
    						replayEvent[propertyXML.@Name.toString()] = new tempClass(tempDate[0], tempDate[1]-1, tempDate[2]);
    					}
    				}
    			}
    		}
    		return null;
		}
		
		/**
		 * Extracts and passes the values of an object that supports
		 * tabulardata (ex DataGrid, List etc) to the testing framework. One can choose what
		 * rows/columns or 0,0 to choose all.
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @param start The start column/row
		 * @param end The end column/row
		 * @return The tabular values in a comma separated list.
		 * 
		 * @see mx.automation.AutomationID
		 */
		public function getTabularDataValues(objectID:String, start:int=0, end:int=0):String
		{
			var object:IAutomationObject = resolveObjFromID(objectID);
			return object.automationTabularData.getValues(start,end);
		}
		
		public function getTabularDataValue(objectID:String, arg:Object):String
		{
			var object:IAutomationObject = resolveObjFromID(objectID);
			var result:Array = object.automationTabularData.getAutomationValueForData(arg);
			return result.join(",");
		}
		
		/**
		 * Returns the index of the first visible row in a component supporting
		 * tabular data e.g. datagrid, list, combobox
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @return The index of the first visible row
		 * 
		 * @see mx.automation.AutomationID
		 * 
		 */
		public function getFirstVisibleRow(objectID:String):int
		{
			var object:IAutomationObject = resolveObjFromID(objectID);
			return object.automationTabularData.firstVisibleRow;
		}
		
		/**
		 * Returns the index of the last visible row in a component supporting
		 * tabular data e.g. datagrid, list, combobox
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @return The index of the last visible row.
		 * 
		 * @see mx.automation.AutomationID
		 */
		public function getLastVisibleRow(objectID:String):int
		{
			var object:IAutomationObject = resolveObjFromID(objectID);
			return object.automationTabularData.lastVisibleRow;
		}
		
		/**
		 * Returns the number of rows in a component supporting
		 * tabular data e.g. datagrid, list, combobox
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @return The number of rows
		 * 
		 * @see mx.automation.AutomationID
		 */
		public function getNumRows(objectID:String):int
		{
			var object:IAutomationObject = resolveObjFromID(objectID);
			return object.automationTabularData.numRows;
		}
		
		/**
		 * Returns the number of columns in a component supporting
		 * tabular data e.g. datagrid, list, combobox
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @return The number of columns
		 * 
		 * @see mx.automation.AutomationID
		 * 
		 */
		public function getNumColumns(objectID:String):int
		{
			var object:IAutomationObject = resolveObjFromID(objectID);
			return object.automationTabularData.numColumns;
		}
		
		/**
		 * Returns the column names of a component supporting
		 * tabular data e.g. datagrid, list, combobox
		 * 
		 * @param objectID The id of the object, String/AutomationID
		 * @return The column names
		 * 
		 * @see mx.automation.AutomationID
		 */
		public function getColumnNames(objectID:String):String
		{
			var object:IAutomationObject = resolveObjFromID(objectID);
			var result:Array = object.automationTabularData.columnNames;
			return result.join(",");
		}
		
		
		/**
		 * Prints the hierarchy of parent to the object in Ruby syntax.
		 * It will be used together with the FunFX recording tool. This method is only created, so
		 * that the adapter is partly ready for the recording tool.
		 * 
		 * @param obj IAutomationObject recorded
		 * @return A string representing the hierarchy
		 * 
		 */
		public static function getHierarchy(obj:IAutomationObject):String
		{
			var parent:UIComponent = UIComponent(obj);
			var event:String;
			var temp:String = parent.className + "(\"" + parent.automationName + "\").";
			while(Automation.automationManager.getParent(parent) != null)
			{
				parent = UIComponent(Automation.automationManager.getParent(parent));
				event = parent.className + "(\"" + parent.automationName + "\").";
				event = event.concat(temp);
				temp = event;
			}
			return event;
		}

		/**
		 * Extracts the information from the recorded event. This method makes it possible to post recorded events to a database.
		 * It is provided to be able to support a recording tool.
		 * 
		 * @param event The recorded event
		 * 
		 */
		private static function recordHandler(event:AutomationRecordEvent):void
	    {
	        Automation.automationManager.incrementCacheCounter();
	
	        try
	        {
	            var obj:IAutomationObject = event.automationObject ;
	            var rid:AutomationID = Automation.automationManager.createID(obj);

	            var httpService:HTTPService = new HTTPService();
	            httpService.url = "http://localhost:3000/testlines/create";
				httpService.method = "POST";
				httpService.useProxy = false;

				var params:Object = new Object();
				params["event"] = getHierarchy(obj) + event.name;

				httpService.send(params);
	            
	        }
	        catch (e:Error)
	        {
	            trace(e.message);
	        }
	
	        Automation.automationManager.decrementCacheCounter();
	    }
		
		/**
		 * Sets the adapter in recording mode
		 */
		public static function beginRecording():void
	    {
	       	Automation.automationManager.addEventListener(AutomationRecordEvent.RECORD, recordHandler, false, 0, true);
	       	Automation.automationManager.beginRecording();
	    }
	
	    /**
	     * Makes the adapter leave the recording mode
	     */
	    public function endRecording():void
	    {
	       	automationManager.endRecording();
	       	automationManager.removeEventListener(AutomationRecordEvent.RECORD, recordHandler);
	    }
		
		/**
		 * Initiates the event on the object.
		 * 
		 * @param obj The display object on which the event is performed on
		 * @param ev The event to be performed
		 * 
		 */
		protected function startPlayback(obj:IAutomationObject, ev:Event):void
	    {
	    	var run:AutomationReplayEvent = new AutomationReplayEvent("replay", false, false, obj, ev);
	    	automationManager.replayAutomatableEvent(run);
	    		
	    }
	    
	    /**
    	 * A getter method to the Application.application.root
    	 * 
    	 * It eases the testing
    	 * 
    	 * @return The application root
    	 * 
    	 */
    	protected function get applicationRoot():Object
    	{
    		return _root;
    	}
		
		/**
		 * A getter method to the Automation.automationManager
		 * 
		 * It eases testing
		 * 
		 * @return The automationManager
		 * 
		 */
		protected function get automationManager():IAutomationManager
	    {
	        return Automation.automationManager ;
	    }
	    
	    
	    /**
	     * Method that checks whether the Flex application is ready or busy doing some action.
	     * @param objectID The id created by AutomationID describing the object
	     * @return Boolean value indicating if the application is synchronized
	     * 
	     * @see mx.automation.AutomationID
	     */
	    public function isSynchronized(objectID:String):Boolean
	    {
	    	return automationManager.isSynchronized(null);
	    }
	    
	    
	    /**
	     * Helper method for displaying the children of a component
	     * Is used to ease writing tests, finding the right children
	     * @param objectID The id of the object, String/AutomationID
	     * @return A String showing all the children with names.
	     * 
	     * @see mx.automation.AutomationID
	     */
	    public function getChildren(objectID:String):String
	    {
	    	if(objectID != null)
	    	{
				var obj:UIComponent = resolveObjFromID(objectID);
					
				var str:String = "AutomationChildren " + obj.numAutomationChildren + " of " + obj.className + " id " + obj.id + " autName " + obj.automationName + ": ";
				for(var i:int=0; i<obj.numAutomationChildren; i++)
					if(obj.getAutomationChildAt(i) is UIComponent)
					{
						str = str + "|" + UIComponent(obj.getAutomationChildAt(i)).className + 
						", automationName: " + UIComponent(obj.getAutomationChildAt(i)).automationName + 
						", id: " + UIComponent(obj.getAutomationChildAt(i)).id +
						", name: " + UIComponent(obj.getAutomationChildAt(i)).name;
					}
					else if(obj.getAutomationChildAt(i) is UITextField)
					{
						str = str + "|" + UITextField(obj.getAutomationChildAt(i)).className + 
						", automationName: " + UITextField(obj.getAutomationChildAt(i)).automationName + 
						", name: " + UITextField(obj.getAutomationChildAt(i)).name;
					}
					else
					{
						str = str + "|" + obj.getAutomationChildAt(i).automationName;
					}
				return str;
	    	}
	    	else
	    	{
	    		str = "AutomationChildren " + DisplayObjectContainer(applicationRoot).numChildren + " of root: ";
	    		for(i=0; i<DisplayObjectContainer(applicationRoot).numChildren; i++)
	    		{
	    			try
	    			{
	    				str = str + "|" + UIComponent(applicationRoot.getChildAt(i)).className +
	    				", automationName: " + UIComponent(DisplayObjectContainer(applicationRoot).getChildAt(i)).automationName +
	    				", id: " + UIComponent(DisplayObjectContainer(applicationRoot).getChildAt(i)).id +
	    				", name: " + UIComponent(DisplayObjectContainer(applicationRoot).getChildAt(i)).name;
	    			}	
	    			catch(e:Error)
	    			{
	    				str = str + "|" + DisplayObjectContainer(applicationRoot).getChildAt(i).name;
	    			}
	    		}
	    		return str;
	    	}
	    }
	}
}