/*
 * FunFXMock is a mock of the FunFX class
 *
 * Copyright (c) 2007 Peter Nicolai Motzfeldt.
 * Licensed under open source. No warranty is provided
 */
package src.test
{
	import src.main.FunFX;
	import flash.display.DisplayObject;
	import mx.automation.IAutomationObject;
	import mx.automation.IAutomationManager;
	import mx.controls.Button;
	import mx.automation.Automation;
	import mx.core.UIComponent;
	import flash.events.Event;

	/**
	 * A mock class of the FunFX class, used for unit testing
	 * @author Peter Nicolai Motzfeldt
	 * 
	 */
	public class FunFXMock extends FunFX
	{
		protected var rootObj:DisplayObject;
		protected var list:Object = new Object();
		
		public function FunFXMock(root:DisplayObject)
		{
			super(root);
			rootObj = root;
			
			setList(UIComponent(rootObj));
		}
		
		private function setList(obj:UIComponent):void
		{
			for(var i:int=0; i<obj.numChildren; i++)
			{
				try
				{
					var child:UIComponent = UIComponent(obj.getChildAt(i));
					list[child.automationName] = IAutomationObject(child);
					setList(child);
				}
				catch(e:Error)
				{}
			}
		}
		
		/**
		 * This method is an override from the FunFX
		 * @return The root object
		 * 
		 */
		override protected function get applicationRoot():Object
    	{
    		return rootObj;
    	}
	    
	    /**
	     * Method that overrides the same method in FunFX, this to be able to test FunFX methods
	     * @param obj The object that the event will be replayed on
	     * @param ev The event that shall be replayed
	     * 
	     */
	    override protected function startPlayback(obj:IAutomationObject, ev:Event):void
	    {
	    	
	    }
	    
	    /**
	     * Overridden method from FunFX
	     * @param objID The object id
	     * @return The object responding to the object id in array form
	     * 
	     */
	    override protected function resolveID(objID:String):Array
	    {
	    	return [list[objID]];
	    }
	    
	   	/**
	   	 * Overridden method from FunFX
	   	 * @param obj The object
	   	 * @return The id created
	   	 * 
	   	 */
	   	override protected function createID(obj:IAutomationObject):String
	   	{
	   		return obj.automationName;
	   	}
	   	
	   	/**
	   	 * Overridden method from FunFX
	   	 * @param openList List of objects not yet investigated
	   	 * @param name The name of the object
	   	 * @return The object found
	   	 * 
	   	 */
	   	override protected function performSearch(openList:Array, name:String):UIComponent
		{
			var closedList:Array = new Array();
			while(openList.length != 0)
			{
				var child:UIComponent  = UIComponent(openList.shift());
 				if(checkName(child, name)) return child;
 				
 				for(var i:int=0; i<child.numChildren; i++)
 				{
 					try
					{
 						openList.push(UIComponent(child.getChildAt(i)));
 					}
	 				catch(e:Error)
	 				{}
 				}
 				
 				closedList.push(child);
 			}
 			return null;
 		}
	}
}