/*
 * FunFXTest is the class containing the unit tests
 *
 * Copyright (c) 2007 Peter Nicolai Motzfeldt.
 * Licensed under open source. No warranty is provided
 */
package src.test
{
	import flexunit.framework.TestCase;
	import flexunit.framework.TestSuite;
	import src.main.FunFX;
	import flexunit.framework.AssertionFailedError;
	import mx.core.Application;
	import flash.events.MouseEvent;
	import flash.events.Event;
	import mx.automation.IAutomationObject;
	import mx.automation.events.AutomationDragEvent;
	import flash.events.TextEvent;
	import flexunit.framework.Assert;
	import mx.controls.Button;

	/**
	 * FunFXTest is the suite containing the unit tests for FunFX
	 * @author Peter Nicolai Motzfeldt
	 * 
	 */
	public class FunFXTest extends TestCase
	{
		private var adapter:FunFX;
		
		public function FunFXTest(methodName:String ) 
		{
   			super( methodName );
        }
        
        public static function suite():TestSuite 
        {
   			var ts:TestSuite = new TestSuite();
   			
   			ts.addTest( new FunFXTest( "testCreate" ) );
   			ts.addTest( new FunFXTest( "testFindObjectNull" ) );
   			ts.addTest( new FunFXTest( "testFindObject" ) );
   			ts.addTest( new FunFXTest( "testCheckComponent" ) );
   			ts.addTest( new FunFXTest( "testCatchProperty" ) );
   			ts.addTest( new FunFXTest( "testCatchEventError" ) );
   			ts.addTest( new FunFXTest( "testSetPropertiesTextInput" ) );
   			ts.addTest( new FunFXTest( "testSetPropertiesMouseScroll" ) );
   			ts.addTest( new FunFXTest( "testSetPropertiesDragStart" ) );
   			ts.addTest( new FunFXTest( "testCheckName" ) );
   			return ts;
   			
   		}
   		
   		/**
   		 * Tests the creation of the adapter
   		 * 
   		 */
   		public function testCreate():void
   		{
   			adapter = new FunFXMock(new DisplayMock());
   			assertNotNull(adapter);
   		}
   		
   		/**
   		 * Tests that the findObject method return null if nothing is found
   		 * 
   		 */
   		public function testFindObjectNull():void
   		{
   			testCreate();
   			
			assertNull(adapter.findObject(null, null));
			assertNull(adapter.findObject(null, "test"));
   		}
   		
   		/**
   		 * Tests the findObject method, locating objects
   		 * 
   		 */
   		public function testFindObject():void 
   		{	
   			testCreate();
   			assertNull(adapter.findObject(null, "test"));
   			assertNotNull(adapter.findObject(null, "cPanel"));
   		}
   		
   		/**
   		 * Test the method checkComponent. Checking whether the objects exists
   		 * 
   		 */
   		public function testCheckComponent():void
   		{
   			testCreate();
   			var value:String = adapter.checkComponent(null, "cPanel");
   			trace("value " + value);
   			assertEquals("cPanel", value)
   			value = adapter.checkComponent(value, "pPanel")
   			assertEquals("pPanel", value)
   			value = adapter.checkComponent(value, "bButton")
   			assertEquals("bButton", value)
   		}
   		
   		/**
   		 * Tests the catchProperty method. Extracting the property of a display object.
   		 * 
   		 */
   		public function testCatchProperty():void
   		{
   			testCreate();
   			var value:String = adapter.checkComponent(null, "cPanel");
   			assertEquals("cPanel", value)
   			value = adapter.checkComponent(value, "pPanel")
   			assertEquals("pPanel", value)
   			value = adapter.checkComponent(value, "bButton")
   			assertEquals("bButton", value)
   			assertEquals("Click", adapter.catchProperty(value, "label"));
   			assertNull(adapter.catchProperty(value, "labell"));
			assertNull(adapter.catchProperty("id", "label"));
			
   		}
   		
   		/**
   		 * Tests the catchEvent method. Ensuring that the correct error messages appears.
   		 * 
   		 */
   		public function testCatchEventError():void
   		{
   			testCreate();
   			assertEquals("No such object", adapter.catchEvent("click", "flash.events::MouseEvent", null));
   			assertNull(adapter.catchEvent("click", "flash.events::MouseEvent", "bButton"));
   			assertEquals("Error #1065: Variable WrongEvent is not defined.", adapter.catchEvent("click", "flash.events::WrongEvent", "bButton"));
   		}
   		
   		/**
   		 * Tests the setProperties method, merging the properties of an event. Tests with text input.
   		 * 
   		 */
   		public function testSetPropertiesTextInput():void
   		{
   			testCreate();
   			var event:TextEvent = new TextEvent("textInput");

   			var properties:Object = "<Event Name='Input'>"
                + "<Implementation Class='flash.events::TextEvent' Type='textInput'/>"
                + "<Property Name='text' DefaultValue='Dette er en test'>"
                + "<PropertyType Type='String'/>"
                + "</Property>"
            	+ "</Event>";
            assertEquals("", event.text);
			var replayEventText:* = adapter.setProperties(event, null, properties);
			assertEquals("Dette er en test", event.text);
   		}
   		
   		/**
   		 * Tests the setProperties method, merging the properties of an event. Tests with mousescroll.
   		 * 
   		 */
   		public function testSetPropertiesMouseScroll():void
   		{
   			testCreate();
   			var event:MouseEvent = new MouseEvent("mouseWheel");

   			var properties:Object = "<Event Name='MouseScroll'>"
                + "<Implementation Class='flash.events::MouseEvent' Type='mouseWheel'/>"
                + "<Property Name='delta'  DefaultValue='5'>"
                + "<PropertyType Type='int'/>"
                + "</Property>"
            	+ "</Event>";
            assertEquals("0", event.delta);
			var replayEventText:* = adapter.setProperties(event, null, properties);
			assertEquals("5", event.delta);
   		}
   		
   		/**
   		 * Tests the setProperties method, merging the properties of an event. Test drag and drop with the DraggedItem.
   		 * 
   		 */
   		public function testSetPropertiesDragStart():void
   		{
   			testCreate();
 
   			var event:AutomationDragEvent = new AutomationDragEvent("dragStart");
			var object:ObjectMock = new ObjectMock();
   			var properties:Object = "<Event Name='DragStart'>"
                + "<Implementation Class='mx.automation.events::AutomationDragEvent' Type='dragStart'/>"
                + "<Property Name='draggedItem' DefaultValue='Test'>"
                + "<PropertyType Type='String' />"
                + "</Property>"
				+ "<Property Name='ctrlKey'  DefaultValue='false'>"
				+ "<PropertyType Type='Boolean' />"
				+ "</Property>"
				+ "<Property Name='shiftKey'  DefaultValue='false'>"
				+ "<PropertyType Type='Boolean' />"
				+ "</Property>"
				+ "<Property Name='altKey'  DefaultValue='false'>"
				+ "<PropertyType Type='Boolean' />"
				+ "</Property>"
            	+ "</Event>";
            assertNull(event.draggedItem);
			var replayEventText:* = adapter.setProperties(event, object, properties);
			assertNotNull(event.draggedItem);
   		}
   		
   		/**
   		 * Tests the checkName method.
   		 * 
   		 */
   		public function testCheckName():void
   		{
   			var testName:String = "testButton";
   			testCreate();
   			var buttonName:Button = new Button();
   			buttonName.id = null;
   			buttonName.name = testName;
   			buttonName.automationName = null;
   			assertTrue(adapter.checkName(buttonName, testName));
   			assertFalse(adapter.checkName(buttonName, "wrong"));
   			
   			var buttonId:Button = new Button();
   			buttonId.id = testName;
   			buttonId.name = "";
   			buttonId.automationName = null;
   			assertTrue(adapter.checkName(buttonId, testName));
   			assertFalse(adapter.checkName(buttonId, "wrong"));
   			
   			var buttonAutName:Button = new Button();
   			buttonAutName.id = null;
   			buttonAutName.name = "";
   			buttonAutName.automationName = testName;
   			assertTrue(adapter.checkName(buttonAutName, testName));
   			assertFalse(adapter.checkName(buttonAutName, "wrong"));
   			
   			var buttonLabel:Button = new Button();
   			buttonLabel.automationName = testName;
   			assertTrue(adapter.checkName(buttonLabel, testName));
   			assertFalse(adapter.checkName(buttonLabel, "wrong"));
   			
   		}
   		
   		
	}
}