# This module contains different browser implementations
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.

require 'win32ole'
require 'conf'

module Browsers
  
  # This class is the Internet Explorer implmentation.
  # It uses the WIN32OLE to be able to drive IE programmatically
  class InternetExplorer
    
    def initialize(visible)
      sleep($sleep_between_tests)
      @ie = WIN32OLE.new('InternetExplorer.Application')
      @ie.visible = visible
    end
    
    # This method navigates the browser to the address, and reaches the
    # embedded Flash/Flex object with the name provided with the method
    # set_flex_object
    def goto(address, name)
      while(busy)
        sleep($sleep_busy)
      end
      navigate(address)
      set_flex_object(name)
      
      #-- Must sleep so IE is ready
      sleep($sleep_wait)
    end
    
    # This method sets the Flash/Flex object, with the help of 
    # help_set_flex_object
    def set_flex_object(name)
      while(busy)
        sleep($sleep_busy)
      end

      @flex = help_set_flex_object(name)
      raise "No flash element by name #{name}" if @flex.item == nil
    end
    
    # This makes the Flash/Flex object available for the Funfx class
    def flex_object
      @flex.item(0)
    end
    
    # Shuts down the browser
    def unload
      while(@ie.nil?)
        sleep($sleep_busy)
      end
      begin
        @ie.Quit
      rescue
      end
    end
    
    # This method makes the framework wait if the browser is busy
    def busy
      begin
        @ie.Busy
      rescue
        return true
      end
    end
    
    # This method loops a defined number of times until it reaches the
    # Flash object. It uses a global variable named -$count_set_flex- from conf.rb
    def help_set_flex_object(name, obj=nil)
      count = $count_set_flex
      obj = help_get_element(name)
      while(obj.item == nil && count > 0)
        sleep(1)
        count = count - 1
        obj = help_get_element(name)
      end
      return obj
    end
    
    # Helper method to extract the Flash object. The argument is the name of the object.
    def help_get_element(name)
      begin
        return @ie.Document.getElementsByName(name)
      rescue
        return nil
      end
    end
    
    # This method navigates the browser to the address provided
    def navigate(address)
      while(@ie.nil?)
        sleep($sleep_busy)
      end
      @ie.Navigate(address)
    end
    
    # This method sets the browser visible or invisible
    def visible(visible)
      @ie.visible = visible
    end
  end
end
