# This class is the facade class of the framework, it is built as a singleton.
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.

require 'singleton'
require 'xml_parser'
require 'browsers'

class Funfx < Flex::Base
  include Singleton
  
  attr_accessor :speed

  def initialize
    XmlParser.parse
    @speed = 0
  end
  
  # This method will parse a XML file, and create new classes. It
  # uses XmlParser
  def parse_file(document)
    XmlParser.parse_file(document)
  end
  
  # This method generates a new browser window, and the visibility is false as default.
  # Internet Explorer is defined as default browser type (and the currently only implemented)
  def start(visible = false, browser_type = "InternetExplorer")
    @browser = Browsers.const_get(browser_type).new(visible)
  end
  
  # Provides the browser with address and name. The browser navigates to the address, and
  # extracts the Flash/Flex object with the provided name
  def goto(address, name)
    @browser.goto(address, name)
  end
  
  # Sets the visibility of the browser
  def visible(visible)
    @browser.visible(visible)
  end
  
  # Unloads the browser
  def unload
    @browser.unload
  end
  
  # Returns the Flash/Flex object
  def get_flex_object
    @browser.flex_object
  end
end
