require 'test/unit'
require 'lib/funfx'
require 'flexmock'
require 'base_test'

# This test case tests the functionality of the Funfx class
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.
class FunfxTest < Test::Unit::TestCase
  
  def setup
    
  end
  
  def teardown

  end
  
  # This tests tests that the instance is not nil
  def test_a_instance
    @instance = Funfx.instance
    assert_not_nil(@instance)
  end
  
  # This test tests that it is possible to reach children
  def test_b_flexobject
    @mockinstance = MockFunfx.instance
    assert_not_nil(@mockinstance)
    assert_not_nil(@mockinstance.button("FlexObject"))
  end
  
  # This test emulates the relative path
  def test_c_object_path
    @mockinstance = MockFunfx.instance
    assert_not_nil(@mockinstance)
    assert_equal("idBuy",@mockinstance.AppMock("app").button("Buy").id)
    assert_nil(@mockinstance.AppMock("app").button("BuyButton"))
  end
  
  
  
  
end

# This class is a mock of the Funfx class.
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.
class MockFunfx < Funfx
  include FlexMock::TestCase
  
  def initialize
    Funfx.instance
  end
  
  def AppMock(name)
    return MockBase.new(name, "idApp")
  end
  
  # Mock method that returns a mock object
  def flex_object
    m = flexmock("funfx")
    m.should_receive(:checkComponent).with("id1","FlexObject").and_return("id2")
    m.should_receive(:checkComponent).with("id2","Button").and_return("id3")
    m.should_receive(:checkComponent).with(nil, "FlexObject").and_return("id1")
    
    m.should_receive(:checkComponent).with(nil, "app").and_return("idApp")
    
    return m
  end
  
  # Mock of the method -is_sync- in Base, that return true
  def is_sync
    true
  end

end

# This class is a mock of the base class.
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.
class MockBase < Flex::Base
    include FlexMock::TestCase
    
    # Mock method of the -ready- method in Base
    def ready
      return true
    end
    
    # Mock method that returns a mock object
    def flex_object
      m = flexmock("funfx")
      m.should_receive(:checkComponent).with("idApp", "Buy").and_return("idBuy")
      m.should_receive(:checkComponent).with("idApp", "BuyButton").and_return(false)
      return m
    end
    
    # Mock of the method -is_sync- in Base, that return true
    def is_sync
      return true
    end
  
  end
